package org.yourorghere;

/** 
 * Manage shapes for my Toy JOGL programs.
 * Deals with shape menu and rendering the shapes.
 */
import java.awt.event.*;
import javax.swing.*;
import javax.media.opengl.GL2;
import javax.media.opengl.glu.*;
import com.jogamp.opengl.util.gl2.*;

public class ShapeManager {

    String[] shapeNames = {"Teapot", "Torus", "Cube", "Cone", "Dodecahedron", "Xwing", "Sphere",
        "Tree", "Function", "Terrain", "Bezier Patch", "Revolution", "CSE Logo"};
    String curShape = shapeNames[0];
    JMenu shape;

    public ShapeManager() {

        shape = new JMenu("Shape");
        for (int i = 0; i < shapeNames.length; i++) {
            JMenuItem mi = new JMenuItem(shapeNames[i]);
            mi.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    curShape = e.getActionCommand();
                }
            });
            shape.add(mi);
        }
    }

    public JMenu getMenu() {
        return shape;
    }

    public void display(GL2 gl, int level, boolean useWireframe) {
        GLU glu = new GLU();
        GLUquadric qobj = glu.gluNewQuadric();
        glu.gluQuadricDrawStyle(qobj, GLU.GLU_FILL);
        glu.gluQuadricTexture(qobj, true);

        GLUT glut = new GLUT();
        //# of slices for spheres and the like based on level
        int slices = 4 * (1 << level); //1<<level is 2^level
        if (curShape.equals("Teapot")) {
            if (useWireframe) {
                glut.glutWireTeapot(1, false);
            } else {
                glut.glutSolidTeapot(1, false); //'false' means normals point out
            }
        } else if (curShape.equals("Torus")) {
            Shapes.torus(gl, 0.75, 1.25, slices, slices);
        } else if (curShape.equals("Cube")) {
            Shapes.cube(gl, 1);
        } else if (curShape.equals("Cone")) {
            glu.gluCylinder(qobj, 1, 0, 1, slices, level + 1);
        } else if (curShape.equals("Xwing")) {
            gl.glPushMatrix();
            gl.glScaled(0.5, 0.5, 0.5);
            StarWars.xwing(gl, slices);
            gl.glPopMatrix();
        } else if (curShape.equals("Sphere")) {
            glu.gluSphere(qobj, 1, slices, slices);
        } else if (curShape.equals("Tree")) {
            gl.glPushMatrix();
            gl.glTranslated(0, -2, 0);
            gl.glScaled(0.4, 0.4, 0.4);
            FractalTree.tree(gl, level);
            gl.glPopMatrix();
        } else if (curShape.equals("Function")) {
            gl.glPushMatrix();
            gl.glTranslated(-0.5, 0, -0.5);
            gl.glScaled(2, 2, 2);
            Function2D.surface(gl, slices);
            gl.glPopMatrix();
        } else if (curShape.equals("Terrain")) {
            gl.glPushMatrix();
            gl.glTranslated(-0.5, 0, -0.5);
            gl.glScaled(2, 2, 2);
            FractalTerrain.surface(gl, slices, 0.2);
            gl.glPopMatrix();
        } else if (curShape.equals("Bezier Patch")) {
            gl.glPushMatrix();
            gl.glTranslated(0, 0, -0.5);
            gl.glScaled(0.6, 0.6, 0.6);
            BezierPatch.patch(gl, slices);
            gl.glPopMatrix();
        } else if (curShape.equals("Revolution")) {
            gl.glPushMatrix();
            gl.glTranslated(0, -1, 0);
            gl.glScaled(2, 2, 2);
            SurfaceOfRevolution.surface(gl, slices);
            gl.glPopMatrix();
        } else if (curShape.equals("CSE Logo")) {
            gl.glPushMatrix();
            gl.glScaled(0.4, 0.4, 0.4);
            CSELogo.logo(gl);
            gl.glPopMatrix();
        }
    }
}
