package org.yourorghere;

/** Functions for drawing shapes like a torus and a cube.
    Replaces ones from glut that don't generate texture coordinates
    Written by Tim Lambert based on code in glut
*/ 


/* Some of this code was copied from glut and hence we have to include
   the notice below */

/* Copyright (c) Mark J. Kilgard, 1994, 1997. */

/**
(c) Copyright 1993, Silicon Graphics, Inc.

ALL RIGHTS RESERVED

Permission to use, copy, modify, and distribute this software
for any purpose and without fee is hereby granted, provided
that the above copyright notice appear in all copies and that
both the copyright notice and this permission notice appear in
supporting documentation, and that the name of Silicon
Graphics, Inc. not be used in advertising or publicity
pertaining to distribution of the software without specific,
written prior permission.

THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU
"AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR
OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  IN NO
EVENT SHALL SILICON GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE
ELSE FOR ANY DIRECT, SPECIAL, INCIDENTAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER,
INCLUDING WITHOUT LIMITATION, LOSS OF PROFIT, LOSS OF USE,
SAVINGS OR REVENUE, OR THE CLAIMS OF THIRD PARTIES, WHETHER OR
NOT SILICON GRAPHICS, INC.  HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH LOSS, HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
ARISING OUT OF OR IN CONNECTION WITH THE POSSESSION, USE OR
PERFORMANCE OF THIS SOFTWARE.

US Government Users Restricted Rights

Use, duplication, or disclosure by the Government is subject to
restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
(c)(1)(ii) of the Rights in Technical Data and Computer
Software clause at DFARS 252.227-7013 and/or in similar or
successor clauses in the FAR or the DOD or NASA FAR
Supplement.  Unpublished-- rights reserved under the copyright
laws of the United States.  Contractor/manufacturer is Silicon
Graphics, Inc., 2011 N.  Shoreline Blvd., Mountain View, CA
94039-7311.

OpenGL(TM) is a trademark of Silicon Graphics, Inc.
*/




import javax.media.opengl.GL2;
import javax.media.opengl.glu.*;


public class Shapes {


    static void	torus(GL2 gl, double r, double R, int nsides, int rings){
	int i, j;
	double theta, phi, theta1;
	double cosTheta, sinTheta;
	double cosTheta1, sinTheta1;
	double ringDelta, sideDelta;
	double s,s1,t,sDelta,tDelta;

	ringDelta = 2.0 * Math.PI / rings;
	sideDelta = 2.0 * Math.PI / nsides;
	sDelta = 1.0 / rings;
	tDelta = 1.0 / nsides;

	theta = 0.0;
	cosTheta = 1.0;
	sinTheta = 0.0;
	s = 0.0;
	for (i = rings - 1; i >= 0; i--) {
	    theta1 = theta + ringDelta;
	    s1 = s + sDelta;
	    cosTheta1 = Math.cos(theta1);
	    sinTheta1 = Math.sin(theta1);
	    gl.glBegin(GL2.GL_QUAD_STRIP);
	    phi = 0.0;
            t = 0.0;
	    for (j = nsides; j >= 0; j--) {
		double cosPhi, sinPhi, dist;
		
		phi += sideDelta;
                t += tDelta;
		cosPhi = Math.cos(phi);
		sinPhi = Math.sin(phi);
		dist = R + r * cosPhi;
		
		gl.glNormal3d(cosTheta1 * cosPhi, -sinTheta1 * cosPhi, sinPhi);
		gl.glTexCoord2d(s1,t);
		gl.glVertex3d(cosTheta1 * dist, -sinTheta1 * dist, r * sinPhi);
		gl.glNormal3d(cosTheta * cosPhi, -sinTheta * cosPhi, sinPhi);
		gl.glTexCoord2d(s,t);
		gl.glVertex3d(cosTheta * dist, -sinTheta * dist,  r * sinPhi);
	    }
	    gl.glEnd();
	    theta = theta1;
            s = s1;
	    cosTheta = cosTheta1;
	    sinTheta = sinTheta1;
	}
    }
    

    static float n[][] =
    {
	{-1.0f, 0.0f, 0.0f},
	{0.0f, 1.0f, 0.0f},
	{1.0f, 0.0f, 0.0f},
	{0.0f, -1.0f, 0.0f},
	{0.0f, 0.0f, 1.0f},
	{0.0f, 0.0f, -1.0f}
    };
    static int faces[][] =
    {
	{0, 1, 2, 3},
	{3, 2, 6, 7},
	{7, 6, 5, 4},
	{4, 5, 1, 0},
	{5, 6, 2, 1},
	{7, 4, 0, 3}
    };
    static void	cube(GL2 gl, double size){
	float v[][] = new float[8][3];
	int i;
	
	v[0][0] = v[1][0] = v[2][0] = v[3][0] = (float)-size / 2;
	v[4][0] = v[5][0] = v[6][0] = v[7][0] = (float)size / 2;
	v[0][1] = v[1][1] = v[4][1] = v[5][1] = (float)-size / 2;
	v[2][1] = v[3][1] = v[6][1] = v[7][1] = (float)size / 2;
	v[0][2] = v[3][2] = v[4][2] = v[7][2] = (float)-size / 2;
	v[1][2] = v[2][2] = v[5][2] = v[6][2] = (float)size / 2;
	
	for (i = 5; i >= 0; i--) {
	    gl.glBegin(GL2.GL_POLYGON);
	    gl.glNormal3fv(n[i],0);
            gl.glTexCoord2f(0,0);
	    gl.glVertex3fv(v[faces[i][0]],0);
            gl.glTexCoord2f(1,0);
	    gl.glVertex3fv(v[faces[i][1]],0);
            gl.glTexCoord2f(1,1);
	    gl.glVertex3fv(v[faces[i][2]],0);
            gl.glTexCoord2f(0,1);
	    gl.glVertex3fv(v[faces[i][3]],0);
	    gl.glEnd();
	}
    }


}
